/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.dashboards;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.TreeMap;

import org.appspy.viewer.client.AbstractPanel;
import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.form.DashboardColumnForm;
import org.appspy.viewer.client.form.DashboardForm;
import org.appspy.viewer.client.form.DashboardParamBindingForm;
import org.appspy.viewer.client.form.DashboardReportForm;
import org.appspy.viewer.client.paramchooser.ParamPropertyGridPanelFactory;
import org.appspy.viewer.client.paramchooser.ParamUtils;

import com.google.gwt.core.client.GWT;
import com.google.gwt.http.client.URL;
import com.google.gwt.user.client.ui.Frame;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.core.Function;
import com.gwtext.client.core.Margins;
import com.gwtext.client.core.RegionPosition;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.Tool;
import com.gwtext.client.widgets.Toolbar;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.NumberField;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.BorderLayout;
import com.gwtext.client.widgets.layout.BorderLayoutData;
import com.gwtext.client.widgets.layout.ColumnLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtext.client.widgets.portal.Portal;
import com.gwtext.client.widgets.portal.PortalColumn;
import com.gwtext.client.widgets.portal.Portlet;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class DashboardPanel extends AbstractPanel {

	protected Portal mPortal = null;
	protected DashboardForm mDashboard = null;
	protected Panel mPropertyPanel = null;
	protected Collection<DashboardReportPortlet> mReportPortlets = new ArrayList<DashboardReportPortlet>();

	public DashboardPanel(ViewerModule viewerModule, DashboardForm report) {

		super(viewerModule);
		mDashboard = report;
		mPropertyPanel = ParamPropertyGridPanelFactory.getPropertyGridPanel(viewerModule, report.getParams());

		setLayout(new BorderLayout());
		setAutoScroll(true);

		Toolbar toolbar = new Toolbar();
		toolbar.addSeparator();
		toolbar.addSpacer();
		toolbar.addFill();

		BorderLayoutData centerLayoutData = new BorderLayoutData(RegionPosition.CENTER);
		centerLayoutData.setMargins(new Margins(5, 5, 0, 5));
		centerLayoutData.setCMargins(new Margins(5, 5, 5, 5));
		centerLayoutData.setMinSize(200);
		centerLayoutData.setMinWidth(200);

		mPortal = new Portal();
		mPortal.setBodyBorder(false);
		mPortal.setAutoScroll(true);
		mPortal.setDraggable(true);
		mPortal.setTitle(mDashboard.getName());
		add(mPortal, centerLayoutData);
		GWT.log("portal added " + mPortal, null);
		TreeMap<Integer, DashboardColumnForm> columnTreeMap = new TreeMap<Integer, DashboardColumnForm>();
		for (DashboardColumnForm dashboardColumn : mDashboard.getColumns()) {
			columnTreeMap.put(dashboardColumn.getIndex(), dashboardColumn);
		}
		for (DashboardColumnForm dashboardColumn : columnTreeMap.values()) {
			addColumn(dashboardColumn);
		}

		refreshReports();

		mPropertyPanel.setTitle("Parameters");
		mPropertyPanel.setCollapsible(true);
		mPropertyPanel.setCollapsed(true);
		mPropertyPanel.setWidth(200);
		mPropertyPanel.addButton(new Button("OK", new ButtonListenerAdapter() {

			@Override
			public void onClick(Button button, EventObject e) {
				refreshReports();
			}
		}));

		BorderLayoutData eastLayoutData = new BorderLayoutData(RegionPosition.EAST);
		eastLayoutData.setMargins(new Margins(5, 0, 5, 5));
		eastLayoutData.setMinWidth(100);
		eastLayoutData.setSplit(true);

		add(mPropertyPanel, eastLayoutData);
	}

	public void refreshReports() {
		for (DashboardReportPortlet portlet : mReportPortlets) {
			portlet.refreshReport();
		}
	}

	public void addColumn(final DashboardColumnForm dashboardColumn) {
		GWT.log("addColumn : " + dashboardColumn, null);
		final PortalColumn portalColumn = new DashboardPortalColumn(dashboardColumn);
		portalColumn.setPaddings(10, 10, 0, 10);

		TreeMap<Integer, DashboardReportForm> reportTreeMap = new TreeMap<Integer, DashboardReportForm>();
		for (DashboardReportForm dashboardReport : dashboardColumn.getDashboardReports()) {
			reportTreeMap.put(dashboardReport.getRowIndex(), dashboardReport);
		}
		for (DashboardReportForm dashboardReport : reportTreeMap.values()) {
			DashboardReportPortlet portlet = new DashboardReportPortlet(dashboardReport);
			mReportPortlets.add(portlet);
			portalColumn.add(portlet);
		}

		double columnWidth = dashboardColumn.getWidth();
		mPortal.add(portalColumn, new ColumnLayoutData(columnWidth / getDashboardWidth()));
	}

	public int getDashboardWidth() {
		int width = 0;

		for (DashboardColumnForm dashboardColumn : mDashboard.getColumns()) {
			width = width + dashboardColumn.getWidth();
		}

		return width;
	}
}

class DashboardReportPortlet extends Portlet {
	protected DashboardReportForm mDashboardReport;
	protected Frame mFrame;

	public DashboardReportPortlet(DashboardReportForm dashboardReport) {
		mDashboardReport = dashboardReport;

		setTitle(dashboardReport.getTitle());
		setHeight(dashboardReport.getHeight());
		setLayout(new FitLayout());
		mFrame = new Frame();
		add(mFrame);
		
		Tool gear = new Tool(Tool.GEAR, new Function() {
			public void execute() {
				final Window w = new Window("Report properties", 200, 100, true, true);
				FormPanel fp = new FormPanel();
				fp.setFrame(true);
				
				final NumberField nf = new NumberField("Height");
				nf.setValue(mDashboardReport.getHeight());
				fp.add(nf, new AnchorLayoutData("100%"));
				
				w.add(fp);
				w.addButton(new Button("OK", new ButtonListenerAdapter(){

					@Override
					public void onClick(Button button, EventObject e) {
						setHeight(nf.getValue().intValue());
						w.destroy();
					}}));
				w.addButton(new Button("Cancel", new ButtonListenerAdapter(){

					@Override
					public void onClick(Button button, EventObject e) {
						w.destroy();
					}}));
				w.show();
			}
		});

		Tool[] tools = new Tool[] { gear };

		setTools(tools);

	}

	public void refreshReport() {
		for (DashboardParamBindingForm paramBinding : mDashboardReport.getDashboardParamBindings()) {
			paramBinding.getReportParam().setDefault(paramBinding.getDashboardParam().getDefault());
		}

		String queryString = ParamUtils.paramsToQueryString(mDashboardReport.getReport().getParams());
		String reportUrl = GWT.getModuleBaseURL() + "../org.appspy.viewer.birt.webapp/run?__report=" + URL.encode(mDashboardReport.getReport().getName()) + queryString
				+ "&__refreshTime=" + new Date().getTime();

		GWT.log("reportUrl = " + reportUrl, null);

		mFrame.setUrl(reportUrl);
	}
}

class DashboardPortalColumn extends PortalColumn {
	protected DashboardColumnForm mDashboardColumn;

	public DashboardPortalColumn(DashboardColumnForm dashboardColumn) {
		mDashboardColumn = dashboardColumn;
	}

	/**
	 * @return the dashboardColumn
	 */
	public DashboardColumnForm getDashboardColumn() {
		return mDashboardColumn;
	}

	/**
	 * @param dashboardColumn
	 *            the dashboardColumn to set
	 */
	public void setDashboardColumn(DashboardColumnForm dashboardColumn) {
		mDashboardColumn = dashboardColumn;
	}
}
